class ShowAccess:
    def __set_name__(self, owner, name):
        self.name = name

    def __get__(self, instance, owner):
        if instance is None:
            return self
        if self.name in instance.__dict__:
            value = instance.__dict__[self.name]
            print(f"Get {self.name} = {value}")
            return value
        else:
            raise AttributeError(f"'{owner.__name__}' object has no attribute '{self.name}'")

    def __set__(self, instance, value):
        instance.__dict__[self.name] = value
        print(f"Set {self.name} = {value}")

    def __delete__(self, instance):
        if self.name in instance.__dict__:
            value = instance.__dict__[self.name]
            print(f"Delete {self.name} = {value}")
            del instance.__dict__[self.name]
        else:
            raise AttributeError(f"'{type(instance).__name__}' object has no attribute '{self.name}'")

if __name__ == "__main__":
    class TestClass:
        attr = ShowAccess()

    obj = TestClass()
    obj.attr = 42
    print(obj.attr)
    del obj.attr


# Задача 1. Итератор, который проходит по числам начиная с 10 и до указанного предела
def range_iterator(limit):
    return iter(range(10, limit + 1))


# Задача 2. Бесконечный итератор
import itertools

# (A) – Числа, кратные 7 и 9
def multiples_of_7_and_9(limit):
    for i in range(limit):
        if i % 7 == 0 and i % 9 == 0:
            yield i

# (B) – Повторить строки 15 раз
def repeat_string_15_times(string):
    return itertools.repeat(string, 15)

# (C) – Повторить значение 38 раз
def repeat_value_38_times(value):
    return itertools.repeat(value, 38)


# Задача 3. Итератор для перестановок и комбинаций

# (A) – Все возможные перестановки списка
def permutations(lst):
    if len(lst) <= 1:
        yield lst
    else:
        for i in range(len(lst)):
            el = lst[i]
            lst_wotel = lst[:i] + lst[i + 1:]
            for p in permutations(lst_wotel):
                yield [el] + p

# (B) – Все возможные комбинации в списке без замены в отсортированном порядке
def combinations(lst, d):
    if d == 0:
        yield []
    elif len(lst) < d:
        return
    else:
        for comb in combinations(lst[1:], d - 1):
            yield [lst[0]] + comb
        for comb in combinations(lst[1:], d):
            yield comb


# Задача 4. Итератор для преобразования фраз
def transform_phrases(phrases):
    for phrase in phrases:
        words = [word for word in phrase.split() if word.lower() != 'of']
        transformed_words = [word[:-1] if word.endswith('s') and len(word) > 1 else word for word in words]
        yield ' '.join(transformed_words)

if __name__ == "__main__":
    print("Числа от 10 до 15:")
    for num in range_iterator(15):
        print(num, end=" ")
    print("\n")

    print("Числа, кратные 7 и 9 до 500:")
    for num in multiples_of_7_and_9(500):
        print(num, end=" ")
    print("\n")

    print("Повтор строки 'OOO' 15 раз:")
    for string in repeat_string_15_times("OOO"):
        print(string)
    print("\n")

    print("Повтор значения 23 38 раз:")
    for value in repeat_value_38_times(23):
        print(value)
    print("\n")

    print("Перестновки массива [1, 2, 3]:")
    for lst in permutations([1, 2, 3]):
        print(lst)
    print("\n")

    print("Комбинации в списке без замены в отсортированном порядке, список - [1, 2, 3], длина - 2:")
    for lst in combinations([1, 2, 3], 2):
        print(lst)
    print("\n")

    print("Итератор для преобразования фраз, фразы - basket Of apples, Group oF dogs, collection OF books, list of tasks:")
    phrases = [
        "basket Of apples",
        "Group oF dogs",
        "collection OF books",
        "list of tasks"
    ]
    for phrase in transform_phrases(phrases):
        print(phrase)
    print("\n")


# Декоратор со скобками и без
import time
from functools import wraps

def timeit(func=None, *, min_seconds=0):
    if func and callable(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            start_time = time.perf_counter()
            result = func(*args, **kwargs)
            end_time = time.perf_counter()
            elapsed_time = end_time - start_time

            print(f"Функция '{func.__name__}' выполнена за {elapsed_time:.6f} секунд")
            return result
        return wrapper

    def decorator(f):
        @wraps(f)
        def wrapper(*args, **kwargs):
            start_time = time.perf_counter()
            result = f(*args, **kwargs)
            end_time = time.perf_counter()
            elapsed_time = end_time - start_time

            if elapsed_time > min_seconds:
                print(f"Функция '{f.__name__}' выполнена за {elapsed_time:.6f} секунд")
            return result
        return wrapper

    return decorator

if __name__ == "__main__":
    @timeit
    def slow_sum(a, b, *, delay):
        time.sleep(delay)
        return a + b

    @timeit(min_seconds=2)
    def slow_mul(a, b, *, delay):
        time.sleep(delay)
        return a * b

    print(slow_sum(1, 2, delay=1))
    print(slow_mul(3, 3, delay=1))


from setuptools import setup, find_packages
setup(
    name='mypackage_manqust',
    version='0.1',
    packages=find_packages(),
    description='mypackage for 11 lab',
    author='Катя',
    author_email='eam49@tpu.ru',
    url='https://example.com/mypackage',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)


# Задача 1. Функция-генератор для ряда Фибоначчи
def fibonacci():
    """Генератор для бесконечного ряда Фибоначчи."""
    a, b = 0, 1
    while True:
        yield a
        a, b = b, a + b

# Задача 2. Генератор геометрической прогрессии
def geom_progress(b, q):
    """Генератор для бесконечной геометрической прогрессии с начальным значением b и знаменателем q."""
    b_n = b
    while True:
        yield b_n
        b_n *= q

# Задача 3. Генератор обратного отсчёта
def countdown(n):
    """Генератор обратного отсчёта от n до 0."""
    n_1 = n
    while n_1 >= 0:
        yield n_1
        n_1 -= 1

if __name__ == "__main__":
    print("Фибоначчи (первые 10 чисел):")
    fib_gen = fibonacci()
    for i in range(10):
        print(next(fib_gen), end=" ")
    print("\n")

    print("Геометрическая прогрессия (первые 10 чисел):")
    geom_gen = geom_progress(3, 3)
    for i in range(10):
        print(next(geom_gen), end=" ")
    print("\n")

    print("Обратный отсчёт от 10 до 0:")
    for i in countdown(10):
        print(i, end=" ")
    print()